-- Addon Manager lib 1.0
-- 
-- Copyright (c) 2014 w33zl
--  
-- Permission is hereby granted, free of charge, to any person
-- obtaining a copy of this software and associated documentation
-- files (the "Software"), to deal in the Software without
-- restriction, including without limitation the rights to use,
-- copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following
-- conditions:
--  
-- The above copyright notice and this permission notice shall be
-- included in all copies or substantial portions of the Software.
--  
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-- OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-- NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-- HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-- FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-- OTHER DEALINGS IN THE SOFTWARE.

if not ADDONS then
	ADDONS = { _RegisteredAddons = {} }
end


--- Creates a new addon class and a empty table
-- @param addonName The unique name of the addon 
-- @param addonVersion A version identifier (typically a string in the format {major}.{minor}.{revision})
-- @param author The name of the addon author
-- @param description A description of the addon
-- @return A new Addon class with registered OnLoad and OnReady events
function ADDONS:Create(addonName, addonVersion, author, description)
	return ADDONS:Register({}, addonName, addonVersion, author, description)
end

--- Gets a registered addon, either from the internal reference table or from the global table. 
--- Note: the addon doesn't need to be explicitly registered using the Addon Manager, any global table matching the name would be returned
-- @param addonName The unique name of the addon 
-- @param silent Optional parameter indicating if an error message should be displayed in case no addon was found
-- @return A addon table (or nil if not found)
function ADDONS:Get(addonName, silent)
	if ADDONS._RegisteredAddons[addonName] then
		return ADDONS._RegisteredAddons[addonName]
	elseif _G[addonName] then
		return _G[addonName]
	elseif not silent then
		d("|cFF0000Could not find any add-on registered with the name '" .. addonName .. "'|r")
	end
end

--- Registers an existing table as a addon class
-- @param T A table to be extended as a Addon class
-- @param addonName The unique name of the addon 
-- @param addonVersion A version identifier (typically a string in the format {major}.{minor}.{revision})
-- @param author The name of the addon author
-- @param description A description of the addon
-- @return A new Addon class based on the existing table (or a new table if existingTable was nil) with registered OnLoad and OnReady events
function ADDONS:Register(T, addonName, addonVersion, author, description)

	if ADDONS:Get(addonName, true) then
		d("|cFF0000A add-on with the name '" .. addonName .. "' is already registered, each add-on must have a unique name. Change to name of the add-on or try using ADDONS:Get(<<addon_name>>) if you want to load existing add-on.|r")
		--error("A add-on with the name '" .. addonName .. "' is already registered, each add-on must have a unique name")
		return nil
	end

	local T = AddonBase:new(T)

	-- Prepare addon object
	if not T.Addon then T.Addon = {} end
	if not T.Addon.__STATE then T.Addon.__STATE = {} end

	-- Set defaults
	T.Addon.__STATE.IsLoaded = T.Addon.__STATE.IsLoaded or false
	T.Addon.__STATE.IsReady = T.Addon.__STATE.IsReady or false

	-- Set name, version etc
	T.Addon.Name = addonName
	T.Addon.Version = addonVersion
	T.Addon.Author = author
	T.Addon.Description = description

	-- Register events
	EVENT_MANAGER:RegisterForEvent(T.Addon.Name, EVENT_ADD_ON_LOADED, function(event, addonName) 
	 	
	 	if addonName == T.Addon.Name then

	 		if T.Addon.__STATE.IsLoaded then return end

	 		local loadCallback = T.OnLoad

	 		 if T.OnLoad and type(T.OnLoad) == "function" then
	 		 	
	 		 	T:OnLoad()

	 		 	-- Make sure run only once
	 		 	T.Addon.__STATE.IsLoaded = true
	 		 end--if
	 	end--if

	end )--event

	EVENT_MANAGER:RegisterForEvent(T.Addon.Name, EVENT_PLAYER_ACTIVATED, function(obj) 
		trace("Fired event PLAYER_ACTIVATED", T)

		if T.Addon.__STATE.IsReady then return end

	 	if T.SupressLoadMessage == false then
	 		d("" .. T.Addon.Name .. " " .. T.Addon.Version .. " loaded!")
	 	end--if

		if T.OnReady and type(T.OnReady) == "function" then
			T:OnReady()

 		 	-- Make sure run only once
 		 	T.Addon.__STATE.IsReady = true			
		end
	 	
	end )--event

	ADDONS._RegisteredAddons[addonName] = T

	return T

end

local __globalTrace = trace

function trace(text, this)
	if not __globalTrace then return end
	if not TRACE then return end

	__globalTrace(text, this)
end

-- Meta class
AddonBase = { SupressLoadMessage = false }

-- Derived class method new
function AddonBase:new(o) --o,length,breadth)
  o = o or {}
  -- local o = {}
  setmetatable(o, self)
  self.__index = self
  return o
end

--- 
function AddonBase:EnableSettings(enableGlobal, optionalCustomName)
	d("Not implemented")
end

-- Derived class method printArea
function AddonBase:OnLoad()
  --d("This is the default method, should be overloaded")
end

function AddonBase:OnReady()
  --d("This is the default method, should be overloaded")
end




